/**
  ******************************************************************************
  * File Name          : main.c
  * Description        : Main program body
  ******************************************************************************
  *
  * COPYRIGHT(c) 2016 STMicroelectronics
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "stm32f4xx_hal.h"
#include "fatfs.h"

/* USER CODE BEGIN Includes */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "user_def.h"

	
/* USER CODE END Includes */

/* Private variables ---------------------------------------------------------*/
I2C_HandleTypeDef hi2c1;
I2C_HandleTypeDef hi2c2;

SD_HandleTypeDef hsd;
HAL_SD_CardInfoTypedef SDCardInfo;

SPI_HandleTypeDef hspi1;

TIM_HandleTypeDef htim3;

UART_HandleTypeDef huart2;

/* USER CODE BEGIN PV */
/* Private variables ---------------------------------------------------------*/
HAL_SD_ErrorTypedef res_sd;

FRESULT res_t;
FILINFO fno_t;
FIL fil_t;
DIR dir_t;
FATFS fs32_t;
uint32_t byteswritten, bytesread;                     /* File write/read counts */
	

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
void Error_Handler(void);
static void MX_GPIO_Init(void);
static void MX_TIM3_Init(void);
static void MX_USART2_UART_Init(void);
static void MX_SPI1_Init(void);
static void MX_I2C1_Init(void);
static void MX_I2C2_Init(void);
static void MX_SDIO_SD_Init(void);
static void MX_NVIC_Init(void);

/* USER CODE BEGIN PFP */
/* Private function prototypes -----------------------------------------------*/
FRESULT scan_files (
    char* path        /* Start node to be scanned (***also used as work area***) */
);
		
void SD_cad_info_CSDType_displayy(void);
void SD_card_Info_print(void);
void SD_Card_stat_display(void);
void f_write_test(void);
void f_read_test(void);
void f_opendir_scan(void);

/* USER CODE END PFP */

/* USER CODE BEGIN 0 */
#ifdef __GNUC__
 #define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
 #define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
#endif
 
PUTCHAR_PROTOTYPE
{
 HAL_UART_Transmit(&huart2, (uint8_t *)&ch, 1, 1);
 return ch;
}

void SD_card_Info_print(void)
{
	printf("CardBlockSize[uint32] : %08X\n\r",SDCardInfo.CardBlockSize);
	printf("CardCapacity[uint64] : %08X:%08X\n\r",(uint32_t)(SDCardInfo.CardCapacity>>32),(uint32_t)(SDCardInfo.CardCapacity>>0));
	printf("CardType[uint8] : %02X\n\r",SDCardInfo.CardType);
	printf("RCA[uint16] : %04X\n\r",SDCardInfo.RCA);
}
void SD_cad_info_CSDType_displayy(void)
{
	printf("################## SDCardInfo.SD_csd #################\n\r");

	printf("CSDStruct[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.CSDStruct);
	printf("SysSpecVersion[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.SysSpecVersion);
	printf("TAAC[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.TAAC);
	printf("NSAC[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.NSAC);
	printf("MaxBusClkFrec[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.MaxBusClkFrec);
	printf("CardComdClasses[__IO uint16_t] : %04X\n\r",SDCardInfo.SD_csd.CardComdClasses);
	printf("RdBlockLen[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.RdBlockLen);
	printf("PartBlockRead[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.PartBlockRead);
	printf("WrBlockMisalign[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.WrBlockMisalign);
	printf("RdBlockMisalign[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.RdBlockMisalign);
	printf("DSRImpl[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.DSRImpl);
	printf("DeviceSize[__IO uint32_t] : %08X\n\r",SDCardInfo.SD_csd.DeviceSize);
	printf("MaxRdCurrentVDDMin[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.MaxRdCurrentVDDMin);
	printf("MaxRdCurrentVDDMax[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.MaxRdCurrentVDDMax);
	printf("MaxWrCurrentVDDMin[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.MaxWrCurrentVDDMin);
	printf("MaxWrCurrentVDDMax[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.MaxWrCurrentVDDMax);
	printf("DeviceSizeMul[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.DeviceSizeMul);
	printf("EraseGrSize[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.EraseGrSize);
	printf("EraseGrMul[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.EraseGrMul);
	printf("WrProtectGrSize[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.WrProtectGrSize);
	printf("WrProtectGrEnable[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.WrProtectGrEnable);
	printf("ManDeflECC[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.ManDeflECC);
	printf("WrSpeedFact[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.WrSpeedFact);
	printf("MaxWrBlockLen[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.MaxWrBlockLen);
	printf("WriteBlockPaPartial[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.WriteBlockPaPartial);
	printf("ContentProtectAppli[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.ContentProtectAppli);
	printf("FileFormatGrouop[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.FileFormatGrouop);
	printf("CopyFlag[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.CopyFlag);
	printf("PermWrProtect[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.PermWrProtect);
	printf("TempWrProtect[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.TempWrProtect);
	printf("FileFormat[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.FileFormat);
	printf("ECC[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.ECC);
	printf("CSD_CRC[__IO uint8_t] : %02X\n\r",SDCardInfo.SD_csd.CSD_CRC);
}

void SD_Card_stat_display(void)
{
		HAL_SD_CardStatusTypedef  CardStatus;
		HAL_SD_ErrorTypedef res_SD;
	
		res_SD = HAL_SD_GetCardStatus(&hsd,&CardStatus);
		// static HAL_SD_ErrorTypedef SD_CmdResp1Error(SD_HandleTypeDef *hsd, uint8_t SD_CMD)  <-- error generated in HAL_SD_GetCardStatus()

	// bus width display
		printf("res = %d, CardStatus.width = %2d\n\r",res_SD,CardStatus.DAT_BUS_WIDTH);
}

void f_write_test(void)
{
	#if _USE_LFN
    TCHAR lfn[_MAX_LFN + 1];
    fno_t.lfname = lfn;
    fno_t.lfsize = sizeof lfn;
	#endif
	
	uint8_t testBuffer[] = "SD write success try1";
  UINT testBytes;

	res_t = f_mount(&fs32_t,SD_Path,1);
	printf("SD Mount : res f_mount : %02X\n\r",res_t);
	

	if (res_t == FR_OK)
	{
    uint8_t path[13] = "testfile.txt";
    path[12] = '\0';
 
    res_t = f_open(&fil_t, (char*)path, FA_WRITE | FA_CREATE_ALWAYS);
 
    res_t = f_write(&fil_t, testBuffer, cus_Size_of((char *)testBuffer), &testBytes);
 
    res_t = f_close(&fil_t);
	}

	res_t = f_mount(0,SD_Path,0);
	printf("SD Unmount : res f_mount : %02X\n\r",res_t);
}

void f_read_test(void)
{
	#if _USE_LFN
    TCHAR lfn[_MAX_LFN + 1];
    fno_t.lfname = lfn;
    fno_t.lfsize = sizeof lfn;
	#endif
	
	BYTE testBuffer[512];
  UINT testBytes;
	DWORD fsk_offset=0;
	UINT cnt_rd_Byte=80;
	
	res_t = f_mount(&fs32_t,SD_Path,1);
	printf("SD Mount : res f_mount : %02X\n\r",res_t);
	
	if (res_t == FR_OK)
	{
    uint8_t path[13] = "MESSAGE.txt";
    path[11] = '\0';
 
    res_t = f_open(&fil_t, (char*)path, FA_READ); 
		printf("res f_open : %02X\n\r",res_t);

		fsk_offset = 0;
		res_t = f_lseek(&fil_t,fsk_offset);
		printf("\n\rres f_lseek : %02X, offset : %10d\n\r",res_t,fsk_offset);
		res_t = f_read(&fil_t, testBuffer, cnt_rd_Byte, &testBytes);
		printf("res f_read : %02X\n\r",res_t);
		printf("%s",testBuffer);

		fsk_offset = 200;
		res_t = f_lseek(&fil_t,fsk_offset);
		printf("\n\rres f_lseek : %02X, offset : %10d\n\r",res_t,fsk_offset);
		res_t = f_read(&fil_t, testBuffer, cnt_rd_Byte, &testBytes);
		printf("res f_read : %02X\n\r",res_t);
		printf("%s",testBuffer);

		fsk_offset = 400;
		res_t = f_lseek(&fil_t,fsk_offset);
		printf("\n\rres f_lseek : %02X, offset : %10d\n\r",res_t,fsk_offset);
		res_t = f_read(&fil_t, testBuffer, cnt_rd_Byte, &testBytes);
		printf("res f_read : %02X\n\r",res_t);
		printf("%s",testBuffer);

		fsk_offset = 600;
		res_t = f_lseek(&fil_t,fsk_offset);
		printf("\n\rres f_lseek : %02X, offset : %10d\n\r",res_t,fsk_offset);
		res_t = f_read(&fil_t, testBuffer, cnt_rd_Byte, &testBytes);
		printf("res f_read : %02X\n\r",res_t);
		printf("%s",testBuffer);
			printf("\n\r");

			res_t = f_close(&fil_t);
	}

	res_t = f_mount(0,SD_Path,0);
	printf("SD Unmount : res f_mount : %02X\n\r",res_t);
}

void f_opendir_scan(void)
{
	#if _USE_LFN
    TCHAR lfn[_MAX_LFN + 1];
    fno_t.lfname = lfn;
    fno_t.lfsize = sizeof lfn;
	#endif
	TCHAR path[200] = "";
	
	res_t = f_mount(&fs32_t,SD_Path,1);
	printf("SD Mount : res f_mount : %02X\n\r",res_t);
	
	if (res_t == FR_OK)
	{
    res_t = f_opendir(&dir_t,path);
		printf("res f_open : %02X\n\r",res_t);
		
		if (res_t == FR_OK)
		{
        while(1)
        {
            char *fn;
             
            res_t = f_readdir(&dir_t, &fno_t);
             
            if (res_t != FR_OK)
                printf("res = %d f_readdir\n\r", res_t);
 
            if ((res_t != FR_OK) || (fno_t.fname[0] == 0))
                break;
 
#if _USE_LFN
      fn = *fno_t.lfname ? fno_t.lfname : fno_t.fname;
#else
      fn = fno_t.fname;
#endif
            printf("%c%c%c%c ",
                ((fno_t.fattrib & AM_DIR) ? 'D' : '-'),
                ((fno_t.fattrib & AM_RDO) ? 'R' : '-'),
                ((fno_t.fattrib & AM_SYS) ? 'S' : '-'),
                ((fno_t.fattrib & AM_HID) ? 'H' : '-') );
 
            printf("%10d ", fno_t.fsize);
             
            printf("%s/%s\n\r", path, fn);
        }		
		}

		res_t = f_mount(0,SD_Path,0);
		printf("SD Unmount : res f_mount : %02X\n\r",res_t);
	}
}

/* USER CODE END 0 */

int main(void)
{

  /* USER CODE BEGIN 1 */
  /* USER CODE END 1 */

  /* MCU Configuration----------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* Configure the system clock */
  SystemClock_Config();

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_TIM3_Init();
  MX_USART2_UART_Init();
  MX_SPI1_Init();
  MX_I2C1_Init();
  MX_I2C2_Init();
  MX_SDIO_SD_Init();
  MX_FATFS_Init();

  /* Initialize interrupts */
  MX_NVIC_Init();

  /* USER CODE BEGIN 2 */
	//uint8_t sd_state = MSD_OK;
	//SD_Card_stat_display();	// error.. no
	res_t = BSP_SD_Init();
	printf("res BSP_SD_Init : %02X\n\r",res_t);
	//SD_Card_stat_display(); // error.. no
	//SD_card_Info_print();
	//SD_cad_info_CSDType_displayy();
	
	printf("sizeof(FATFS) : %4d\n\r",sizeof(FATFS));
	memset(&fs32_t, 0, sizeof(FATFS));

	printf("Write Test\n\r");
	f_write_test();
	
	printf("Read dir Test\n\r");
	f_opendir_scan();
	
	printf("Read Test\n\r");
	f_read_test();

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
  /* USER CODE END WHILE */

  /* USER CODE BEGIN 3 */

  }
  /* USER CODE END 3 */

}

/** System Clock Configuration
*/
void SystemClock_Config(void)
{

  RCC_OscInitTypeDef RCC_OscInitStruct;
  RCC_ClkInitTypeDef RCC_ClkInitStruct;

  __HAL_RCC_PWR_CLK_ENABLE();

  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);

  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_BYPASS;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = 4;
  RCC_OscInitStruct.PLL.PLLN = 84;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
  RCC_OscInitStruct.PLL.PLLQ = 4;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;
  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
  {
    Error_Handler();
  }

  HAL_SYSTICK_Config(HAL_RCC_GetHCLKFreq()/1000);

  HAL_SYSTICK_CLKSourceConfig(SYSTICK_CLKSOURCE_HCLK);

  /* SysTick_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(SysTick_IRQn, 0, 0);
}

/** NVIC Configuration
*/
static void MX_NVIC_Init(void)
{
  /* TIM3_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(TIM3_IRQn, 2, 0);
  HAL_NVIC_EnableIRQ(TIM3_IRQn);
  /* USART2_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(USART2_IRQn, 6, 0);
  HAL_NVIC_EnableIRQ(USART2_IRQn);
}

/* I2C1 init function */
static void MX_I2C1_Init(void)
{

  hi2c1.Instance = I2C1;
  hi2c1.Init.ClockSpeed = 100000;
  hi2c1.Init.DutyCycle = I2C_DUTYCYCLE_2;
  hi2c1.Init.OwnAddress1 = 0;
  hi2c1.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  hi2c1.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  hi2c1.Init.OwnAddress2 = 0;
  hi2c1.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  hi2c1.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  if (HAL_I2C_Init(&hi2c1) != HAL_OK)
  {
    Error_Handler();
  }

}

/* I2C2 init function */
static void MX_I2C2_Init(void)
{

  hi2c2.Instance = I2C2;
  hi2c2.Init.ClockSpeed = 100000;
  hi2c2.Init.DutyCycle = I2C_DUTYCYCLE_2;
  hi2c2.Init.OwnAddress1 = 0;
  hi2c2.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  hi2c2.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  hi2c2.Init.OwnAddress2 = 0;
  hi2c2.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  hi2c2.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  if (HAL_I2C_Init(&hi2c2) != HAL_OK)
  {
    Error_Handler();
  }

}

/* SDIO init function */
static void MX_SDIO_SD_Init(void)
{

  hsd.Instance = SDIO;
  hsd.Init.ClockEdge = SDIO_CLOCK_EDGE_RISING;
  hsd.Init.ClockBypass = SDIO_CLOCK_BYPASS_DISABLE;
  hsd.Init.ClockPowerSave = SDIO_CLOCK_POWER_SAVE_DISABLE;
  hsd.Init.BusWide = SDIO_BUS_WIDE_1B;
  hsd.Init.HardwareFlowControl = SDIO_HARDWARE_FLOW_CONTROL_DISABLE;
  hsd.Init.ClockDiv = 0;

}

/* SPI1 init function */
static void MX_SPI1_Init(void)
{

  hspi1.Instance = SPI1;
  hspi1.Init.Mode = SPI_MODE_MASTER;
  hspi1.Init.Direction = SPI_DIRECTION_2LINES;
  hspi1.Init.DataSize = SPI_DATASIZE_8BIT;
  hspi1.Init.CLKPolarity = SPI_POLARITY_LOW;
  hspi1.Init.CLKPhase = SPI_PHASE_1EDGE;
  hspi1.Init.NSS = SPI_NSS_SOFT;
  hspi1.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_2;
  hspi1.Init.FirstBit = SPI_FIRSTBIT_MSB;
  hspi1.Init.TIMode = SPI_TIMODE_DISABLE;
  hspi1.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  hspi1.Init.CRCPolynomial = 10;
  if (HAL_SPI_Init(&hspi1) != HAL_OK)
  {
    Error_Handler();
  }

}

/* TIM3 init function */
static void MX_TIM3_Init(void)
{

  TIM_ClockConfigTypeDef sClockSourceConfig;
  TIM_MasterConfigTypeDef sMasterConfig;

  htim3.Instance = TIM3;
  htim3.Init.Prescaler = 1000;
  htim3.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim3.Init.Period = 1000;
  htim3.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  if (HAL_TIM_Base_Init(&htim3) != HAL_OK)
  {
    Error_Handler();
  }

  sClockSourceConfig.ClockSource = TIM_CLOCKSOURCE_INTERNAL;
  if (HAL_TIM_ConfigClockSource(&htim3, &sClockSourceConfig) != HAL_OK)
  {
    Error_Handler();
  }

  sMasterConfig.MasterOutputTrigger = TIM_TRGO_RESET;
  sMasterConfig.MasterSlaveMode = TIM_MASTERSLAVEMODE_DISABLE;
  if (HAL_TIMEx_MasterConfigSynchronization(&htim3, &sMasterConfig) != HAL_OK)
  {
    Error_Handler();
  }

}

/* USART2 init function */
static void MX_USART2_UART_Init(void)
{

  huart2.Instance = USART2;
  huart2.Init.BaudRate = 115200;
  huart2.Init.WordLength = UART_WORDLENGTH_8B;
  huart2.Init.StopBits = UART_STOPBITS_1;
  huart2.Init.Parity = UART_PARITY_NONE;
  huart2.Init.Mode = UART_MODE_TX_RX;
  huart2.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart2.Init.OverSampling = UART_OVERSAMPLING_16;
  if (HAL_UART_Init(&huart2) != HAL_OK)
  {
    Error_Handler();
  }

}

/** Configure pins as 
        * Analog 
        * Input 
        * Output
        * EVENT_OUT
        * EXTI
*/
static void MX_GPIO_Init(void)
{

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOH_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  None
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler */
  /* User can add his own implementation to report the HAL error return state */
  while(1) 
  {
  }
  /* USER CODE END Error_Handler */ 
}

#ifdef USE_FULL_ASSERT

/**
   * @brief Reports the name of the source file and the source line number
   * where the assert_param error has occurred.
   * @param file: pointer to the source file name
   * @param line: assert_param error line source number
   * @retval None
   */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
    ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */

}

#endif

/**
  * @}
  */ 

/**
  * @}
*/ 

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
